@@ -1,8 +1,11 @@
 # -*- coding: utf-8 -*-
 
+from __future__ import print_function
 from importlib import import_module
 import re
 
+from yadic.util import deep_merge
+
 
 class Injectable(type):
     "Provides the __init__ with the suitable args, based on dependencies"
@@ -41,30 +44,16 @@ class Container(object):
         """Rebuilds the configuration for the speedup purpose
         :param config: initial configuration
         :type config: dict"""
-        def apply_if(a, b):
-            for bk, bv in b.iteritems():
-                av = a.get(bk)
-                if av is None:
-                    a[bk] = bv
-                else:
-                    av_is_dict, bv_is_dict = (
-                        isinstance(av, dict), isinstance(bv, dict))
-                    if av_is_dict and bv_is_dict:
-                        apply_if(av, bv)
-                    elif av_is_dict or bv_is_dict:
-                        raise ValueError(
-                            'Structures of plan and cusomization must match!')
-                    else:
-                        a[bk] = bv
-            return a
-
         result = {}
-        for sect, elems in config.iteritems():
+        for sect, elems in config.items():
             plan = elems.get('__default__', {})
             section = result[sect] = {}
-            for el_name, customization in elems.iteritems():
+            for el_name, customization in elems.items():
                 if el_name != '__default__':
-                    section[el_name] = apply_if(plan.copy(), customization)
+                    section[el_name] = deep_merge(
+                        plan.copy(), customization,
+                        lambda x, y, m, p: y
+                    )
         return result
 
     @staticmethod
@@ -127,7 +116,7 @@ class Container(object):
                 result = self._singletones.get((group, name))
             if not is_singleton or not result:
                 deps = {}
-                for dep_group, dep_name in blueprint.iteritems():
+                for dep_group, dep_name in blueprint.items():
                     if dep_group.startswith('_'):
                         continue
                     elif dep_group.startswith('$'):
@@ -152,13 +141,13 @@ class Container(object):
 
         is_ident = re.compile(r'(?i)^[a-z]\w*$').match
 
-        for group, elems in cfg.iteritems():
+        for group, elems in cfg.items():
             if not is_ident(group):
                 wrong('group', (group,))
-            for el, cfg in elems.iteritems():
+            for el, cfg in elems.items():
                 if not is_ident(el) and el != '__default__':
                     wrong('element', (group, el))
-                for k, v in cfg.iteritems():
+                for k, v in cfg.items():
                     if not (
                         is_ident(k) or
                         (is_ident(k[1:]) and k[0] == '$') or
@@ -199,13 +188,17 @@ if __name__ == '__main__':
         def __str__(self):
             return "%s on %s" % (self.name, self.fuel)
 
-    class Vehicle(object):
-        __metaclass__ = Injectable
-        depends_on = ('actuator', 'engine')
-
-        def __str__(self):
-            return "The vehicle, driven by %s, which powered by %s" % (
-                self.actuator, self.engine)
+    # 2/3 compatible class
+    Vehicle = Injectable(
+        'Vehicle',
+        (object,),
+        {
+            'depends_on': ('actuator', 'engine'),
+            '__str__': lambda self: (
+                "The vehicle, driven by %s, which powered by %s" % (
+                    self.actuator, self.engine))
+        }
+    )
 
     entities = {
         'demo.veh.Vehicle': Vehicle,
@@ -254,8 +247,8 @@ if __name__ == '__main__':
     )(config)
 
     for name, _, realization in cont.itergroup('vehicle'):
-        print '%s :: %s' % (name, realization.__name__)
-        print '  ', cont.get('vehicle', name)
+        print('%s :: %s' % (name, realization.__name__))
+        print('  ', cont.get('vehicle', name))
 
     # ===========================================
     # static elements
@@ -317,4 +310,3 @@ if __name__ == '__main__':
         }
     })
     assert cont.get('results', 'sum_x_y') == 42
-
